@@ -11,10 +11,11 @@ def load_file(file_path):
 
 @click.command(help='Usually just `gpt-repl @thread-name`')
 @click.argument("cmd", type=str, required=False)
-@click.option('-f', '--file', type=click.Path(exists=True), help="Path to text file to preload")
+@click.option('-m', '--mode', default='synth-chat', help="REPL mode to load")
 @click.option('-t', '--thread', default=None, help="Thread name to open")
+@click.option('-f', '--file', type=click.Path(exists=True), help="Path to text file to preload")
 @click.option('-l', '--list', is_flag=True, help="List all threads")
-def run(cmd, file, thread, list):
+def run(cmd, mode, thread, file, list):
   config_path = os.getenv('GPT_REPL_CONFIG_PATH', '~/.config/gpt_repl')
 
   if str(cmd) == 'help':
@@ -50,6 +51,7 @@ def run(cmd, file, thread, list):
         break
 
   repl = REPL(
+    mode_name=mode,
     thread_id=thread,
     config_path=config_path,
     autofills=autofills,
