@@ -1,6 +1,6 @@
 import os, re, pyperclip
 
-from .printer import printer
+from .utils import printer
 from .publish import PublishGPT
 
 class CommandsClass:
@@ -14,12 +14,6 @@ class CommandsClass:
     if text == '':
       pass
 
-    elif text.startswith('.>'):
-      sh = text[2:].strip()
-      self.print_command_banner(f'.> {sh}')
-      os.system(sh)
-      print()
-
     elif text == '.clear' or text == '.cl':
       self.clear_screen()
 
@@ -57,6 +51,11 @@ class CommandsClass:
       print(f"Thread name is \"{cli.thread['id']}\"")
       print()
 
+    elif text.startswith('.seed') or text.startswith('.s ') or text == '.s':
+      self.print_command_banner('.seed')
+      cli.mode.set_seed(' '.join(text.split(' ')[1:]).strip())
+      print('Seed updated\n')
+
     elif text.startswith('.rename'):
       self.print_command_banner('.rename')
 
@@ -71,7 +70,7 @@ class CommandsClass:
 
       print()
 
-    elif text == '.reset' or text == '.r':
+    elif text == '.reset' or text == '.rs':
       cli.reset()
       self.clear_screen()
 
@@ -79,12 +78,19 @@ class CommandsClass:
       print(f"@{cli.thread['id']} has been reset")
       print()
 
-    elif text.startswith('.save'):
-      cli.save_thread()
+    elif text == '.redo' or text == '.retry' or text == '.r':
+      self.print_command_banner('.redo')
+      history = cli.thread['history']
+      if len(history) < 2:
+        print('No message to redo\n')
+        return (text, True)
 
-      self.print_command_banner('.save')
-      print(f"@{cli.thread['id']} has been saved")
-      print()
+      cli.mode.rollback()
+      cli.mode.rollback()
+      last_message = history[-2]['text']
+      cli.thread['history'] = history[:-2]
+
+      return (last_message, False)
 
     elif text == '.help' or text == 'help':
       self.print_command_banner('.help')
@@ -97,11 +103,11 @@ class CommandsClass:
         print(thread)
 
     elif text == '.debug' or text == '.db':
-      cli.Chat.debug()
+      cli.mode.debug()
 
     elif text == '.print' or text == '.pp':
       self.print_command_banner('.print')
-      cli.Chat.print()
+      cli.mode.print()
       print()
 
     elif text[0] == '.':
@@ -109,9 +115,9 @@ class CommandsClass:
       print(f"Invalid command. Use .help to list valid commands.\n")
 
     else:
-      return False
+      return (text, False)
 
-    return True
+    return (text, True)
 
   def print_command_banner(self, name, newline=True):
     if newline:
@@ -124,7 +130,6 @@ class CommandsClass:
   def print_help(self):
     printer.print_markdown("""
 ## Commands
-**.>**: Executes everything after '.>' as a shell command.
 **.clear** or .cl: Clears the terminal screen.
 **.copy** or .cp: Copies the first code block in the most recent response to the clipboard.
 **.debug** or .db: Triggers a breakpoint in the GPT backend
@@ -133,10 +138,12 @@ class CommandsClass:
 **.help** or help: Prints a list of available commands and a brief description of each.
 **.print** or .pp: Prints the internal GPT prompt representation
 **.publish** or .pub: Publishes the current thread online and prints the URL.
-**.rename**: Renames the current thread. The new name must be provided as an argument to this command, e.g. ".rename my_new_name"
-**.reset**: or .r: Resets the history of the thread.
+**.redo**: or .r: Resubmits the most recent successful response.
+**.rename**: Renames the current thread. The new name must be provided as an argument to this command, e.g. ".rename my_new_name". The old thread isn't deleted.
+**.reset** or .rs: Resets the history of the thread.
 **.thread** or .name: Prints the current thread name.
 **.threads**: Lists all threads.
+**.seed** or .s: The the text following .seed will be set as the conversation seed. Set no text to clear the seed.
 ## Shortcuts
 **Enter**: Submits the current message.
 **Tab**: Adds a new line.
