@@ -1,86 +1,80 @@
-import prompt_toolkit as PromptToolkit
+import sys
+
 from prompt_toolkit import PromptSession
 from prompt_toolkit.key_binding import KeyBindings
 from prompt_toolkit.history import FileHistory
-from prompt_toolkit.formatted_text import HTML
 from prompt_toolkit.application.current import get_app
 from prompt_toolkit import filters as Filters
 
-from .utils import peek
+from .utils import peek, printer, Loader
 from .config import Config
-from .printer import printer
 
-from .modes.synth_chat import SynthChatMode as ChatBackend
-from .spinner import Spinner
+# from .modes.synth_chat import SynthChatMode as ReplMode
+from .modes import get_mode
 from .commands import Commands
 
 class REPL:
 
-  def __init__(self, thread_id=None, config_path="~/.config/gpt_repl", autofills=[]):
+  def __init__(
+      self,
+      thread_id=None,
+      config_path="~/.config/gpt_repl",
+      autofills=[],
+      mode_name = 'synth-chat'
+    ):
 
     self.config = Config(config_path)
     self.thread = self.config.load_thread(thread_id)
     self.autofills = autofills
-    self.Chat = None
+    self.mode_name = mode_name
+
+    self.first_run = True
 
   def run(self):
     self.warmup()
-    self.Chat = ChatBackend( state=self.thread['state'] )
+    self.load_mode(self.mode_name)
 
     print("Enter 'help' for a list of commands. Use Enter to submit and Tab to start a new line.\n")
     self.replay_thread()
     self.start_prompt_session()
 
-    first_run = True
-
     while True:
       try:
         self.print_you_banner(len(self.thread["history"]) + 1)
+        printer.pad_down(3)
 
-        if len(self.autofills) == 0:
-          text = self.prompt()
-        else:
-          text = self.autofills.pop(0)
+        default = ''
+        if len(self.autofills) > 0:
+          default = self.autofills.pop(0)
 
+        text = self.prompt(default=default)
         printer.print_markdown(text)
 
-        if Commands.exec(self, text):
+        ( text, stop ) = Commands.exec(self, text)
+        if stop:
           continue
 
-        self.thread["history"].append({
-          "type": "you",
-          "text": text,
-        })
-
-        # hack, will fix when I feel like it
         print('')
-        if first_run:
-          self.print_gpt_banner(len(self.thread["history"]) + 1)
-          first_run = False
-        else:
-          self.print_gpt_banner(len(self.thread["history"]) + 1, stats=self.Chat.stats())
+        self.print_gpt_banner(len(self.thread["history"]) + 2, stats=self.mode.stats())
+        printer.pad_down(3)
 
-        with Spinner(show_timer=True, delay=1.5) as spinner:
-          chat_data = peek(self.Chat.ask(text))[0]
-
-        answer = ''
-        with printer.live(transient=True) as screen:
-          for data in chat_data:
-            answer += data
-            markdown = printer.to_markdown(answer.lstrip() + '█', code_theme='default')
-            display_text = markdown.to_text() + '\n\n\n'
-            screen.update(display_text)
-        answer = answer.strip()
+        try:
+          answer = self.ask(text)
+        except (KeyboardInterrupt, EOFError):
+          self.mode.rollback()
+          print('Request canceled\n')
+          continue
 
         printer.print_markdown(answer)
         print('')
 
-        self.thread["history"].append({
-          "type": "gpt",
-          "text": answer,
-        })
+        self.thread["history"] += [
+          { "type": "you", "text": text },
+          { "type": "gpt", "text": answer }
+        ]
 
         self.save_thread()
+        self.first_run = False
       except (KeyboardInterrupt, EOFError):
         self.save_thread()
         printer.print_thread_closed(self.thread['id'])
@@ -89,7 +83,9 @@ class REPL:
       except Exception as e:
         breakpoint()
         printer.print_thread_closed(self.thread['id'])
-        raise e
+
+        printer.exception(e)
+        sys.exit(1)
 
   def start_prompt_session(self):
     self.session = PromptSession(
@@ -116,13 +112,21 @@ class REPL:
       else:
         event.current_buffer.insert_text('\n')
 
-  def prompt(self):
+  def prompt(self, default=''):
+    seed = self.mode.get_seed()
+    if seed:
+      bottom_toolbar = f'seed={seed}'
+    else:
+      bottom_toolbar = 'No conversation seed set'
+
     text = self.session.prompt(
       '',
       multiline=True,
       key_bindings=self.kb,
       enable_open_in_editor=True,
       tempfile_suffix='.md',
+      default=default,
+      bottom_toolbar=bottom_toolbar,
     )
     return text.strip()
 
@@ -131,13 +135,13 @@ class REPL:
       bg_color='rgb(0,95,135)',
       text=' You:',
       prefix=f' {count} ',
-      suffix=f' @{self.thread["id"]} [ synth-chat-mode ]'
+      suffix=f' @{self.thread["id"]} [ {self.mode_name} ]'
     )
 
   def print_gpt_banner(self, count, stats=''):
     printer.print_banner(
       bg_color='spring_green4',
-      text=' GPT:',
+      text=f' {self.mode.title}:',
       prefix=f' {count} ',
       suffix=stats
     )
@@ -148,12 +152,20 @@ class REPL:
     self.config.save_thread(self.thread['id'], {
       "id": self.thread['id'],
       "history": self.thread["history"],
-      'state': self.Chat.save(),
+      'state': self.mode.save(),
     })
 
+  def load_mode(self, mode_name):
+    self.mode = get_mode(mode_name)(state=self.thread['state'])
+
   def reset(self):
+    current_seed = self.mode.get_seed() if self.mode else ''
+
     self.thread = self.config.get_empty_thread(self.thread['id'])
-    self.Chat = ChatBackend(state=self.thread['state'])
+
+    self.load_mode(self.mode_name)
+    self.mode.set_seed(current_seed)
+
     self.save_thread()
 
   def replay_thread(self):
@@ -170,3 +182,19 @@ class REPL:
     if any([ '```' in m for m in messages ]):
       with printer.print_thread_loading(self.thread['id']):
         printer.preload()
+
+  def ask(self, text):
+    delay = 0.25 if self.first_run else 1.5
+    with Loader(show_timer=True, delay=delay) as spinner:
+      gen = iter(self.mode.ask(text))
+      response = peek(gen)[0]
+
+    answer = ''
+    with printer.live(transient=True) as screen:
+      for data in response:
+        answer += data
+        markdown = printer.to_markdown(answer.lstrip() + '█', code_theme='default')
+        display_text = markdown.to_text() + '\n\n\n'
+        screen.update(display_text)
+
+    return answer.strip()
